clear

%% Set-up
% We use the following notation: 
% m1, m2, m3 are the angular wavenumbers
% n1, n2, n3 are the radial wavenumber indices
% k1, k2, k3 are the radial wavenumbers, satifying the no-flux condition

% The triad conditions for a circular cylinder are are as follows:
% m1 + m2 = m3;     om1 + om2 = om3;

% We consider |m_j| <= m_max and n_j <= n_max for j = 1, 2, 3.
% We further restrict our attention with the following conditions:
% 1. m3 >= 0 (WLOG by swapping the signs of m1 and m2) 
% 2. m1 <= m2 (WLOG by swapping the roles of m1 and m2)

% Note: the conditions m1 + m2 = m3 with m3 >= 0 and m1 <= m2 imply that 
% m1 <= m3/2 and m2 >= m3/2, which we use to bound the "for" loop below.

% Note: as m2 <= m_max and m1 = m3 - m2, we find that m1 >= m3 - m_max. We
% use this bound in the "for" loop below.

% By combining the above bounds, we find that m3 - m_max <= m1 <= m3/2.
% We combine an outer loop over all values of 0 <= m3 <= m_max, and an 
% inner loop over m3 - m_max <= m1 <= m3/2. For given m1 and m3, we then
% define m2 = m3 - m1.

% Define the values of m_max >= 1 and n_max >= 1 (m_max and n_max do not 
% need to be the same value). The larger the values of m_max and n_max, 
% the more triads are generated (and the slower the code). Be prepared to
% wait a few minutes if m_max and n_max are large (e.g. = 20).

% In table 1 of the main text, we use m_max = 3 and n_max = 3.
m_max = 3; % <<< CHANGE ME >>>
n_max = 3; % <<< CHANGE ME >>>

%% Define the angular frequency for given depth, h, and wavenumber, k
omf = @(h, k) sqrt(k * tanh(k * h));

%% Storage 
k1_vec = [];    k2_vec = [];    k3_vec = [];    % radial wavenumbers
m1_vec = [];    m2_vec = [];    m3_vec = [];    % angular wavenumbers
n1_vec = [];    n2_vec = [];    n3_vec = [];    % radial wavenumber indices
h_vec  = [];                                    % critical depths

%% Loop through all possible combinations, determining resonances
for m3 = 0:m_max
    disp(['m3 = ' num2str(m3)])
    for m1 = (m3 - m_max):(m3/2) % guarantees that m1 <= m2
        m2 = m3 - m1;
        
        % 1. Determine all candidate radial wavenumbers.
        % Note: zerobess returns k = 0 for m = 0, which we remove as want
        % want the first n_max NON-ZERO wavenumbers.
        % --- m1 ---
        if m1 == 0
            k1_all = zerobess('DJ', abs(m1), n_max+1); k1_all(1) = [];
        else
            k1_all = zerobess('DJ', abs(m1), n_max);
        end
        
        % --- m2 ---
        if m2 == 0
            k2_all = zerobess('DJ', abs(m2), n_max+1); k2_all(1) = [];
        else
            k2_all = zerobess('DJ', abs(m2), n_max);
        end
        
        % --- m3 ---
        if m3 == 0
            k3_all = zerobess('DJ', abs(m3), n_max+1); k3_all(1) = [];
        else
            k3_all = zerobess('DJ', abs(m3), n_max);
        end
        
        % 2. Loop through candidate triads for given k1, k2, k3
        for n3 = 1:length(k3_all)
            k3 = k3_all(n3);            
            
            for n1 = 1:length(k1_all)
                k1 = k1_all(n1);
                
                for n2 = 1:length(k2_all)
                    k2 = k2_all(n2);
                    
                    % 3. Determine if there is a resonance (see Theorem 1)
                    if logical( ((k1+k2) < k3) .* (k3 < (sqrt(k1)+sqrt(k2))^2) ) 
                        
                        % 4. Find critical depth, h (wide search range)
                        % Define function for finding the critical depth
                        depth_func = @(h) (omf(h,k1) + omf(h,k2))./omf(h,k3) - 1;
                        
                        h = fzero(@(h) depth_func(h), [1e-8, 100]);
                        
                        % 5. Storage (append to previously found values)
                        k1_vec = [k1_vec k1];   %#okay
                        k2_vec = [k2_vec k2];   %#okay
                        k3_vec = [k3_vec k3];   %#okay
                        
                        m1_vec = [m1_vec m1];   %#okay
                        m2_vec = [m2_vec m2];   %#okay
                        m3_vec = [m3_vec m3];   %#okay
                        
                        n1_vec = [n1_vec n1];   %#okay
                        n2_vec = [n2_vec n2];   %#okay
                        n3_vec = [n3_vec n3];   %#okay
                        
                        h_vec  = [h_vec h];     %#okay
                    end
                end
            end  
        end
    end
end

%% Define the mean wavenumber involved in the triad
kmean = 1/3 * (k1_vec + k2_vec + k3_vec);

%% Compute correlation integrals
% Unnormalised Bessel functions involved in each triad
B1_vec = @(r) besselj(m1_vec, k1_vec*r);
B2_vec = @(r) besselj(m2_vec, k2_vec*r);
B3_vec = @(r) besselj(m3_vec, k3_vec*r);

% Normalisation constant for each Bessel function
N1_vec = sqrt( (1 - m1_vec.^2./k1_vec.^2) .* B1_vec(1).^2 );
N2_vec = sqrt( (1 - m2_vec.^2./k2_vec.^2) .* B2_vec(1).^2 );
N3_vec = sqrt( (1 - m3_vec.^2./k3_vec.^2) .* B3_vec(1).^2 );

% Normalised Bessel functions
C1 = @(r) B1_vec(r)./N1_vec;
C2 = @(r) B2_vec(r)./N2_vec;
C3 = @(r) B3_vec(r)./N3_vec;

% Compute correlation integral, C = (1/pi) \iint Psi_1 Psi_2 Psi_3^* dA
% Note: the correlation integral differs from equation (4.7) as we consider
% om1 + om2 = om3, rather than om1 + om2 + om3 = 0
disp('--- compute correlation integrals ---')
C_int = 2 * integral(@(r) r .* C1(r) .* C2(r) .* C3(r), 0, 1, 'ArrayValued', true);

%% Optional. Define beta coefficients (from weakly nonlinear analysis)
% We map om3 --> -om3 so that om1 + om2 + om3 = 0 (as used in the weakly 
% nonlinear analysis)

% om1_vec = +sqrt(k1_vec .* tanh(k1_vec .* h_vec));
% om2_vec = +sqrt(k2_vec .* tanh(k2_vec .* h_vec));
% om3_vec = -sqrt(k3_vec .* tanh(k3_vec .* h_vec));
% 
% beta_vec = (om1_vec .* k1_vec.^2 + om2_vec .* k2_vec.^2 + om3_vec .* k3_vec.^2) - ...
%     0.5 * om1_vec .* om2_vec .* om3_vec .* (om1_vec.^2 + om2_vec.^2 + om3_vec.^2);

%% Scatter plot of triads in existence region
% The dot colour is determned by k3*h_c, i.e. the normalised critical depth
figure
scatter(k1_vec./k3_vec, k2_vec./k3_vec, 10, log10(k3_vec.*h_vec), 'filled')
hold on

% add on symmetric counterpart (swapping roles of m1 and m2) ??
scatter(k2_vec./k3_vec, k1_vec./k3_vec, 10, log10(k3_vec.*h_vec), 'filled')

% add on boundaries
fplot(@(x) 1 - x, [0 1], 'k', 'LineWidth', 4)
fplot(@(x) (1 - sqrt(x)).^2, [0 1], 'k', 'LineWidth', 4)
hold off

colorbar
grid on; box on
axis equal
axis([0 1 0 1])
set(gca, 'FontSize', 20, 'FontName', 'Times New Roman')
xlabel('$$K_1/K_3$$', 'Interpreter', 'Latex', 'FontSize', 22)
ylabel('$$\frac{K_2}{K_3}$$', 'Interpreter', 'Latex', 'FontSize', 22)
set(get(gca, 'YLabel'), 'Rotation', 0)

%% Plot h versus the index number (largest to smallest critical depth)
% Some values of h are repeated when the same triad is generated, but with
% the roles of triads 1 and 2 swapped. It may be more informative to plot
% the unique values instead, i.e. [~,ind]=sort(unique(h_vec, 'descend'));
figure
[~,h_ind] = sort(h_vec, 'descend');

plot(h_vec(h_ind), 'k.')
set(gca, 'FontSize', 20, 'FontName', 'Times New Roman')
xlabel('Index', 'Interpreter', 'Latex', 'FontSize', 22)
ylabel('$$h_c$$', 'Interpreter', 'Latex', 'FontSize', 22)
set(get(gca, 'YLabel'), 'Rotation', 0)

grid on; box on

%% Optional. Histogram over critical depths on log horizontal scale.
% This plot is useful when considering large values of m_max and n_max
% figure
% hist(log10(h_vec), 30)
% set(gca, 'FontSize', 20, 'FontName', 'Times New Roman')
% xlabel('$$\log_{10}h$$', 'Interpreter', 'Latex', 'FontSize', 22)
% grid on
% box on

%% Optional. Plot h versus the product C_int * beta.
% This quantity gives the coefficients of the weakly nonlinear analysis
% figure
% [~,ind] = sort(h_vec, 'ascend');
% 
% semilogy(h_vec(ind), abs(C_int(ind).*beta_vec(ind)), 'k.') % maybe plot unique values instead
% set(gca, 'FontSize', 20, 'FontName', 'Times New Roman')
% xlabel('$$h$$', 'Interpreter', 'Latex', 'FontSize', 22)
% ylabel('$$C \beta$$', 'Interpreter', 'Latex', 'FontSize', 22)
% set(get(gca, 'YLabel'), 'Rotation', 0)
% 
% grid on; box on

%% Display solutions in a table (similar to table 1)
% The 1/pi normalisation factor is removed from the correlation integral 
% for the results presented in table 1 (so we multiply C_int by pi).
% We order the triads in terms of decreasing critical depth.

% "true" means show all critical depths, including repetitions.
% "false" means show only the unique critical depths, like in table 1.
show_repeated_triads = false; % <<< CHANGE ME >>> 

% Sort the selected triads, deepest to shallowest
if show_repeated_triads
    [~,ind] = sort(h_vec, 'descend');
else
    [~, ind, ~] = unique(h_vec, 'sorted');
    ind = flipud(ind);
end

% Create table storing the data, and add column headings
output_table = table(m1_vec(ind).', m2_vec(ind).', m3_vec(ind).', ...
    n1_vec(ind).', n2_vec(ind).', n3_vec(ind).', ...
    k1_vec(ind).', k2_vec(ind).', k3_vec(ind).', kmean(ind).', ...
    h_vec(ind).', pi*C_int(ind).');
output_table.Properties.VariableNames = {'m1', 'm2', 'm3', 'n1', 'n2', 'n3', ...
    'k_1', 'k_2', 'k_3', 'k_mean', 'h_c', 'pi_times_C'};

% Choose how may triads to display in the table 
max_number_to_display = 30;    % <<< CHANGE ME >>>
number_of_triads_to_display = min(max_number_to_display, length(h_vec(ind))); 

% Display the first "number_of_triads_to_display" rows of the table
head(output_table, number_of_triads_to_display)

% Output some summarising information.
if show_repeated_triads
    disp('The table DOES NOT REMOVE repeated critical depths')
else
    disp('The table REMOVES repeated critical depths')
end
disp(['The table shows the triads with the ' ...
    num2str(number_of_triads_to_display) '/' num2str(length(h_vec(ind))) ...
    ' largest critical depths'])
